
/**************************************************************************
 * Copyright (C) 2007 Ruben Pollan Bella <meskio@amedias.org>             *
 *                                                                        *
 *  This file is part of TuDu.                                            *
 *                                                                        *
 *  TuDu is free software; you can redistribute it and/or modify          *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 3 of the License.        *
 *                                                                        *
 *  TuDu is distributed in the hope that it will be useful,               *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include "config.h"

/*
 * Context in the config file
 * define with [ keys ]
 */
#define C_NULL 0
#define C_KEYS 1
#define C_GENERAL 2

bool Config::load(const char* path)
{
	int context = C_NULL;

	ifstream file(path);
	if (!file) return false;

	while (!file.eof())
	{
		char line[256];
		string str = "", option, value;
		int pos;
		bool is_definition = false;

		file.getline(line,256);

		// drop all the spaces
		for (unsigned int i = 0; i<strlen(line); i++)
		{
			if (line[i] == '#') break; //is a coment
			if (line[i] == '=') is_definition = true;
			if ((line[i] != ' ') && (line[i] != '\t'))
				str += line[i];
		}

		/*
		 * Change of context
		 */
		if ((str[0] == '[') && (str[str.length()-1] == ']'))
		{
			string aux = str.substr(1,str.length()-2);
			context = getContext(aux);
			continue;
		}
		if (!is_definition) continue; //is not a valid line

		pos = str.find("=");
		option = str.substr(0, pos);
		value = str.substr(pos+1);
		if (value.empty()) continue;
		switch (context)
		{
			case C_GENERAL:
				getGeneralOption(option, value);
				break;
			case C_KEYS:
				insertKeyMap(tree_keys, option, value);
				break;
		}
	}

	file.close();
	return true;
}

void Config::getGeneralOption(string& option, string& value)
{
	if ("collapse" == option)
	{
		if ("yes" == value)
		{
			collapse = true;
		}
		if ("no" == value)
		{
			collapse = false;
		}
	}
	if ("days_warn" == option)
	{
		days_warn_deadline = atoi(value.c_str());
	}
}

void Config::insertKeyMap(key_map& k, string action, string keys)
{
	if ((keys.length() == 1) || (keys[1] == '#'))
	{
		key_action act;
		act.action = action;
		k.insert(pair<char,key_action>(keys[0],act));
	}
	else
	{
		if (0 == k.count(keys[0]))
		{
			key_action act;
			act.action = "";
			k.insert(pair<char,key_action>(keys[0],act));
		}
		insertKeyMap(k[keys[0]].subkeys, action, keys.substr(1));
	}
}

bool Config::getAction(char key, string& action)
{
	static key_map* key_comb = NULL;

	if (key_comb)
	{
		if (key_comb->count(key) == 1)
		{
			key_action& act = (*key_comb)[key];
			if (!act.subkeys.empty())
			{
				key_comb = &act.subkeys;
			}
			else
			{
				key_comb = NULL;
			}
			if ( act.action != "")
			{
				action = act.action;
				return true;
			}
			else
			{
				action = "";
				return false;
			}
		}
		else
		{
			key_comb = NULL;
		}
	}
	if (tree_keys.count(key) == 1)
	{
		key_action& act = tree_keys[key];
		if (!act.subkeys.empty())
		{
			key_comb = &act.subkeys;
		}
		if ( act.action != "")
		{
			action = act.action;
			return true;
		}
	}

	action = "";
	return false;
}

void Config::getActionList(action_list& list)
{
	_getActionList(list,tree_keys,"");
}

void Config::_getActionList(action_list& list, key_map& k, string key)
{
	key_map::iterator it;

	for (it  = k.begin(); it != k.end(); ++it)
	{
		if (it->second.action != "")
		{
			list[it->second.action] = key + it->first;
		}
		if (!(it->second.subkeys.empty()))
		{
			_getActionList(list, it->second.subkeys, key+it->first);
		}
	}
}

int Config::getContext(string& str)
{
	int context;

	if ("keys" == str)
	{
		context = C_KEYS;
	}
	else if ("general" == str)
	{
		context = C_GENERAL;
	}
	else
	{
		context = C_NULL;
	}

	return context;
}

bool Config::getCollapse()
{
	return collapse;
}

int Config::getDaysWarn()
{
	return days_warn_deadline;
}
