
/**************************************************************************
 * Copyright (C) 2007 Ruben Pollan Bella <meskio@amedias.org>             *
 *                                                                        *
 *  This file is part of TuDu.                                            *
 *                                                                        *
 *  TuDu is free software; you can redistribute it and/or modify          *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 3 of the License.        *
 *                                                                        *
 *  TuDu is distributed in the hope that it will be useful,               *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include "data.h"

ToDo::ToDo(string t, bool isDone, string txt) 
		: title(t), _done(isDone)
{
	collapse = false;
	cursor_in = false;
	text = txt;
}

ToDo::~ToDo()
{
	list<pToDo>::iterator it;
	for (it = childs.begin(); it != childs.end(); it++)
	{
		delete (*it);
	}
}


string& ToDo::getTitle()
{
	return title;
}

Text& ToDo::getText()
{
	return text;
}

Date& ToDo::deadline()
{
	return _deadline;
}

bool& ToDo::done()
{
	return _done;
}

bool ToDo::haveChild()
{
	return !childs.empty();
}

int ToDo::percent()
{
	//FIXME: there are too many tree search -> cache
	if (!haveChild())
	{
		return _done? 100: 0;
	}
	else
	{
		iToDo i(*this);
		int sum_percent = 0, j;
		for (j = 0; !(i.end()); ++i, ++j)
		{
			sum_percent += i->percent();
		}
		return sum_percent/j;
	}
}

bool& ToDo::getCollapse()
{
	return collapse;
}

bool& ToDo::actCollapse()
{
	return cursor_in;
}


#define parent parents.top()

iToDo::iToDo(ToDo& t)
{
	cursor = t.childs.begin();
	parents.push(&t);
}

iToDo& iToDo::operator=(const iToDo& t)
{
	path = t.path;
	parents = t.parents;
	cursor = t.cursor;
	return *this;
}

bool iToDo::operator++()
{
	if (!end())
	{
		++cursor;
		return true;
	}
	else
	{
		return false;
	}
}

bool iToDo::operator--()
{
	if (!begin())
	{
		--cursor;
		return true;
	}
	else
	{
		return false;
	}
}

bool iToDo::in()
{
	if (!end())
	{
		parents.push(*cursor);
		path.push(cursor);
		cursor = (*cursor)->childs.begin();
		return true;
	}
	else
	{
		return false;
	}
}

bool iToDo::out()
{
	if (!path.empty())
	{
		cursor = path.top();
		path.pop();
		parents.pop();
		return true;
	}
	else
	{
		return false;
	}
}

ToDo &iToDo::operator*()
{
	if (!end()) return *(*cursor);
	else 
	{
		return *(parent);
	}
}

ToDo *iToDo::operator->()
{
	if (!end()) return *cursor;
	else 
	{
		return parent;
	}
}

bool iToDo::end()
{
	return ((parent->childs.end() == cursor) && (parent->childs.end()-- == cursor));
}

bool iToDo::begin()
{
	return (parent->childs.begin() == cursor);
}

int iToDo::percentUp()
{
	return parent->percent();
}

void iToDo::addChild(pToDo p)
{
	if (!end()) ++cursor;
	cursor = parent->childs.insert(cursor, p);
}

void iToDo::addChildUp(pToDo p)
{
	cursor = parent->childs.insert(cursor, p);
}

void iToDo::del()
{
	cursor = parent->childs.erase(cursor);
}

int iToDo::depth()
{
	return path.size();
}
