
/**************************************************************************
 * Copyright (C) 2007 Ruben Pollan Bella <meskio@amedias.org>             *
 *                                                                        *
 *  This file is part of TuDu.                                            *
 *                                                                        *
 *  TuDu is free software; you can redistribute it and/or modify          *
 *  it under the terms of the GNU General Public License as published by  *
 *  the Free Software Foundation; either version 3 of the License.        *
 *                                                                        *
 *  TuDu is distributed in the hope that it will be useful,               *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *  GNU General Public License for more details.                          *
 *                                                                        *
 *  You should have received a copy of the GNU General Public License     *
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 **************************************************************************/

#include "interface.h"

#define COLOR_SELECTED COLOR_PAIR(1)
#define COLOR_HELP     COLOR_PAIR(2)
#define COLOR_DEADLINE COLOR_PAIR(3)

#define DEADLINE_COL (COLS-12)

#define cursor_line()  (cursor->line-tree_begin)
#define deadline_close() ((cursor->deadline().valid()) && \
		(cursor->deadline().daysLeft() <= config.getDaysWarn()))

Interface::Interface(iToDo &t, Config &c, Writer &w) 
		: cursor(t), config(c), writer(w), copied(NULL)
{
	/*
	 * Initialize ncurses
	 */
	initscr ();
	if (has_colors()) //TODO: molaria el color en una clase
	{
		start_color();
		init_pair(1,COLOR_GREEN,COLOR_BLACK);
		init_pair(2,COLOR_WHITE,COLOR_BLUE);
		init_pair(3,COLOR_RED,COLOR_BLACK);
	}
	clear ();
	noecho();
	keypad(stdscr, TRUE);
	cbreak(); //TODO: currate la salida
	curs_set(0);
	refresh();

	/*
	 * Create windows
	 */
	whelp = new Window(1, COLS, 0, 0);
	whelp->_attron(COLOR_HELP);
	for (int i = 0; i<COLS; ++i) whelp->_addch(' ');
	whelp->_move(0,0);
	whelp->_addstr(" q:quit  k:up  j:down  h:out  l:in  m:done  o:add  a:modify");
	whelp->_move(0, COLS-8);
	whelp->_addstr("?:help");
	whelp->_attroff(COLOR_HELP);
	whelp->_refresh();
	wtree = new Window(TREE_LINES, COLS, 2, 0);
	tree_begin = 0; tree_end = TREE_LINES;
	wtext = new Window(WTEXT_LINES, WTEXT_COLS, TREE_LINES+2, 1);
	winfo = new Window(1, COLS, LINES-1, 0);
}

Interface::~Interface()
{
	delete whelp;
	delete wtree;
	delete wtext;
	delete winfo;
	delete copied;
	clear ();
	refresh ();
	endwin ();
}

void Interface::main()
{
	int key;
	string action;

	while (cursor.out());
	while (--cursor);
	drawTodo();
	while (true) 
	{
		key = getch();
		infoClear();
		if (KEY_LEFT == key) left();
		if (KEY_RIGHT == key) right();
		if (KEY_DOWN  == key) down();
		if (KEY_UP == key) up();
		if (config.getAction(key, action))
		{
			if ("quit" == action)
			{
				writer.save();
				break;
			}
			if ("quitNoSave" == action) break;
			if ("out" == action) left();
			if ("in" == action) right();
			if ("down" == action) down();
			if ("up" == action) up();
			if ("move_down" == action) move_down();
			if ("move_up" == action) move_up();
			if ("delete" == action) del();
			if ("delDeadline" == action) delDeadline();
			if ("paste" == action) paste();
			if ("pasteUp" == action) pasteUp();
			if ("done" == action) done();
			if ("addTodo" == action) addLine();
			if ("addTodoUp" == action) addLineUp();
			if ("editTitle" == action) modifyLine();
			if ("editDeadline" == action) editDeadline();
			if ("editText" == action) editText();
			if ("downText" == action) downText();
			if ("upText" == action) upText();
			if ("collapse" == action) collapse();
			if ("save" == action) save();
			if ("help" == action) help();
		}
	}
}

void Interface::drawTitle()
{
	int percent = cursor->percent();

	wtree->_move(cursor_line(), 0);
	wtree->_addstr("   ");
	for (int i = 0; i < cursor.depth(); ++i) wtree->_addstr("    ");
	if (cursor->done())
	{
		wtree->_addstr("[X] ");
	}
	else if (0 == percent)
	{
		wtree->_addstr("[ ] ");
	}
	else if (100 == percent)
	{
		wtree->_addstr("100 ");
	}
	else
	{
		char str[8];
		sprintf(str, "%2d%% ", percent);
		wtree->_addstr(str);
	}
	if (cursor->haveChild()) wtree->_attron(A_BOLD);
	wtree->_addstr(cursor->getTitle());
	if (cursor->haveChild()) wtree->_attroff(A_BOLD);

	/* draw date */
	Date& deadline = cursor->deadline();
	if (deadline.valid())
	{
		char s[11];
		wtree->_move(cursor_line(), DEADLINE_COL);
		sprintf(s, "%02d/%02d/%04d", deadline.day(), 
				deadline.month(), deadline.year());
		wtree->_addstr(s);

		if (deadline_close())
		{
			wtree->_attron(COLOR_DEADLINE);
			wtree->_addstr("<-");
			wtree->_attroff(COLOR_DEADLINE);
		}
		else
		{
			wtree->_addstr("  ");
		}
	}

	wtree->_refresh();
}

void Interface::infoMsg(const char str[])
{
	infoClear();
	winfo->_addstr(0,0,str);
	winfo->_refresh();
}

void Interface::infoClear()
{
	winfo->_move(0,0);
	for (int i = 0; i < COLS-15; ++i)
	{
		winfo->_addch(' ');
	}
	winfo->_refresh();
}

void Interface::drawTodo()
{
	iToDo aux = cursor;
	wtree->_erase();
	wtree->_move(0,0);
	while (cursor.out());
	while (--cursor);
	
	char percent[8];
	sprintf(percent, "(%3d%%)", cursor.percentUp());
	winfo->_addstr(0,COLS-7,percent);
	winfo->_refresh();

	int line_counter = 0;
	_drawTodo(line_counter);
	cursor = aux;
	drawCursor();
}

void Interface::_drawTodo(int& line_counter)
{
	for (; !(cursor.end()); ++cursor)
	{
		cursor->line = line_counter++;
		if ((line_counter > tree_begin) && (line_counter <= tree_end))
			drawTitle();
		if ((!cursor->getCollapse()) || (cursor->actCollapse()))
		{
			cursor.in();
			_drawTodo(line_counter);
			cursor.out();
		}
	}
}

void Interface::eraseCursor()
{
	wtree->_move(cursor_line(), 0);
	drawTitle();
}

void Interface::drawCursor()
{
	if (cursor->line > tree_end - 4)
	{
		// FIXME: maybe with scroll is enought
		tree_end = cursor->line + 4;
		tree_begin = tree_end - TREE_LINES;
		drawTodo();
	}
	else if ((tree_begin != 0) && (cursor->line < tree_begin + 4))
	{
		if (cursor->line < 4)
			tree_begin = 0;
		else
			tree_begin = cursor->line - 4;
		tree_end = tree_begin + TREE_LINES;
		drawTodo();
	}
	else
	{
		wtree->_move(cursor_line(), 0);
		wtree->_attron(COLOR_SELECTED);
		drawTitle();
		wtree->_attroff(COLOR_SELECTED);

		wtext->_erase();
		cursor->getText().print(*wtext);
	}
}

void Interface::left()
{
	eraseCursor();
	cursor.out();
	cursor->actCollapse() = false;
	if (cursor->getCollapse())
	{
		drawTodo();
	}
	else
	{
		drawCursor();
	}
}

void Interface::right()
{
	bool need_drawTodo = cursor->getCollapse();

	eraseCursor();
	cursor->actCollapse() = true;
	cursor.in();
	if (cursor.end())
	{
		cursor.addChild(new ToDo());
		drawTodo();
		string title;
		if (editLine(title))
		{
			cursor->getTitle() = title;

			/*
			 * Use the config collapse
			 */
			cursor.out();
			cursor->getCollapse() = config.getCollapse();
			cursor.in();
		}
		else
		{
			del();
		}
	}
	if (need_drawTodo)
	{
		drawTodo();
	}
	else
	{
		drawCursor();
	}
}

void Interface::up()
{
	eraseCursor();
	--cursor;
	drawCursor();
}

void Interface::down()
{
	eraseCursor();
	++cursor;
	if (cursor.end())
	{
		--cursor;
	}
	drawCursor();
}

void Interface::move_up()
{
	pToDo t = &(*cursor);
	cursor.del();
	eraseCursor();
	--cursor;
	cursor.addChildUp(t);
	drawTodo();
}

void Interface::move_down()
{
	pToDo t = &(*cursor);
	cursor.del();
	eraseCursor();
	cursor.addChild(t);
	drawTodo();
}

void Interface::done()
{
	if (cursor->done())
		cursor->done() = false;
	else
		cursor->done() = true;
	drawTodo();
}

void Interface::del()
{
	if (copied) delete copied;
	copied = &(*cursor);
	cursor.del();
	--cursor;
	if ((cursor.end()) && (cursor.begin()))
	{
		cursor.out();
	}
	//TODO: it's not necesary redraw it
	drawTodo();
}

void Interface::delDeadline()
{
	cursor->deadline().year() = 1900;
	wtree->_move(cursor_line(), DEADLINE_COL);
	wtree->_addstr("            ");
	wtree->_refresh();
}

void Interface::paste()
{

	if (copied)
	{
		cursor.addChild(copied);
		copied = NULL;
	}
	drawTodo();
}

void Interface::pasteUp()
{
	cursor.addChildUp(copied);
	drawTodo();
}

#define startTitle (cursor.depth() * 4 + 7)

bool Interface::editLine(string& str)
{
	bool save;

	str = cursor->getTitle();
	infoMsg("Editing todo. Press enter for save it or esc for not save");
	wtree->_attron(COLOR_SELECTED);
	if (cursor->haveChild()) wtree->_attron(A_BOLD);
	lineEditor.getText() = str; 
	lineEditor.cursorPos() = str.length();
	save = lineEditor.edit(*wtree, cursor_line(), startTitle, DEADLINE_COL-startTitle-1);
	if (!save)
	{
		wtree->_move(cursor_line(), startTitle);
		wtree->_addstr(str);
		for (int i = startTitle + str.length(); i < DEADLINE_COL-startTitle-1; i++) 
				wtree->_addch(' ');
		wtree->_refresh();
	}
	str = lineEditor.getText(); 
	if (cursor->haveChild()) wtree->_attroff(A_BOLD);
	wtree->_attroff(COLOR_SELECTED);
	infoClear();
	return save;
}

void Interface::editDeadline()
{
	bool save;
	char date[11] = "00/00/2000";
	Date& deadline = cursor->deadline();
	if (deadline.valid())
	{
		sprintf(date, "%02d/%02d/%02d", deadline.day(),
				deadline.month(), deadline.year());
	}

	infoMsg("Editing deadline. Press enter for save it or esc for not save");
	wtree->_attron(COLOR_SELECTED);

	dateEditor.getText() = date;
	dateEditor.cursorPos() = 0;
	save = dateEditor.edit(*wtree, cursor_line(), DEADLINE_COL, 10);

	if ((!save) && (!deadline.valid()))
	{
		wtree->_move(cursor_line(), DEADLINE_COL);
		wtree->_addstr("          ");
	}
	wtree->_attroff(COLOR_SELECTED);

	/* store deadline */
	if (save)
	{
		strncpy(date, dateEditor.getText().c_str(), 10);
		date[2] = '\0';
		cursor->deadline().day() = atoi(date);
		date[5] = '\0';
		cursor->deadline().month() = atoi(date+3);
		cursor->deadline().year() = atoi(date+6);
	}

	wtree->_move(cursor_line(), COLS-2);
	if (deadline_close())
	{
		wtree->_attron(COLOR_DEADLINE);
		wtree->_addstr("<-");
		wtree->_attroff(COLOR_DEADLINE);
	}
	else
	{
		wtree->_addstr("  ");
	}
	wtree->_refresh();
	infoClear();
}

void Interface::addLine()
{

	cursor.addChild(new ToDo());
	drawTodo();
	string title;
	if (editLine(title))
		cursor->getTitle() = title;
	else
		del();
}

void Interface::addLineUp()
{
	cursor.addChildUp(new ToDo());
	drawTodo();
	string title;
	if (editLine(title))
		cursor->getTitle() = title;
	else
		del();
}

void Interface::modifyLine()
{
	string title;
	if (editLine(title))
		cursor->getTitle() = title;
}

void Interface::editText()
{
	infoMsg("Editing text. Press esc for save it");
	cursor->getText().edit(*wtext);
	infoClear();
}

void Interface::upText()
{
	wtext->_erase();
	cursor->getText().scroll_up();
	cursor->getText().print(*wtext);
}

void Interface::downText()
{
	wtext->_erase();
	cursor->getText().scroll_down();
	cursor->getText().print(*wtext);
}

void Interface::collapse()
{
	cursor->getCollapse() = !cursor->getCollapse();
	drawTodo();
}

void Interface::save()
{
	writer.save();
	infoMsg("File saved");
}

void Interface::help()
{
	action_list list;
	config.getActionList(list);

	Window h(LINES-8, COLS-16, 4, 8);

	string str;
	h._addstr("\n");
	str = "  " + list["out"] + "\tmove out\n";
	h._addstr(str);
	str = "  " + list["in"] + "\tmove in\n";
	h._addstr(str);
	str = "  " + list["down"] + "\tmove down\n";
	h._addstr(str);
	str = "  " + list["up"] + "\tmove up\n";
	h._addstr(str);
	str = "  " + list["move_down"] + "\tmove a todo down\n";
	h._addstr(str);
	str = "  " + list["move_up"] + "\tmove a todo up\n";
	h._addstr(str);
	str = "  " + list["done"] + "\tmark or unmark as done\n";
	h._addstr(str);
	str = "  " + list["delete"] + "\tdelete line\n";
	h._addstr(str);
	str = "  " + list["delDeadline"] + "\tdelete deadline\n";
	h._addstr(str);
	str = "  " + list["paste"] + "\tpaste the last deleted\n";
	h._addstr(str);
	str = "  " + list["pasteUp"] + "\tpaste the last deleted upper than the cursor\n";
	h._addstr(str);
	str = "  " + list["addTodo"] + "\tadd line\n";
	h._addstr(str);
	str = "  " + list["addTodoUp"] + "\tadd line upper than the cursor\n";
	h._addstr(str);
	str = "  " + list["editTitle"] + "\tmodify line\n";
	h._addstr(str);
	str = "  " + list["editText"] + "\tedit text\n";
	h._addstr(str);
	str = "  " + list["editDeadline"] + "\tedit/add deadline\n";
	h._addstr(str);
	str = "  " + list["downText"] + "\tscroll down the text\n";
	h._addstr(str);
	str = "  " + list["upText"] + "\tscroll up the text\n";
	h._addstr(str);
	str = "  " + list["collapse"] + "\tcollapse childs\n";
	h._addstr(str);
	str = "  " + list["save"] + "\tsave todo\n";
	h._addstr(str);
	str = "  " + list["help"] + "\tshow help\n";
	h._addstr(str);
	str = "  " + list["quit"] + "\tquit\n";
	h._addstr(str);
	str = "  " + list["quitNoSave"] + "\tquit without save\n";
	h._addstr(str);
	string s = "--- Pulse any key for close ---";
	h._move(LINES-10, (COLS/2)-8-(s.length()/2));
	h._addstr(s);
	h._box();
	h._getch();
	h._erase();
	h._refresh();
	drawTodo();
}
